/**
 * Copyright (c) 2007, 2010, 2013, 2021 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 * Alexander Shatalin (Borland) - initial API and implementation
 * Artem Tikhomirov             - refactoring of containerBaseCanonicalMethods.xpt; extraction of API/non-API of CEP templates
 * Michael Golubev (Montages) - #386838 - migrate to Xtend2
 * Etienne Allogo (ARTAL) - etienne.allogo@artal.fr - Bug 569174 : 1.4 Merge papyrus extension templates into codegen.xtend
 */
package impl.diagram.update;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import java.util.Arrays;
import java.util.Set;
import metamodel.MetaModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenFeature;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenChildLabelNode;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenCommonBase;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenContainerBase;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenDiagram;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenDiagramUpdater;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenLink;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenNode;
import org.eclipse.papyrus.gmf.codegen.gmfgen.LinkModelFacet;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import xpt.Common;
import xpt.Common_qvto;
import xpt.diagram.updater.DiagramUpdater;
import xpt.diagram.updater.LinkDescriptor;
import xpt.diagram.updater.NodeDescriptor;
import xpt.diagram.updater.Utils_qvto;
import xpt.editor.VisualIDRegistry;

@Singleton
@SuppressWarnings("all")
public class CanonicalUpdate {
  @Inject
  @Extension
  private Common _common;
  
  @Inject
  @Extension
  private Common_qvto _common_qvto;
  
  @Inject
  @Extension
  private Utils_qvto _utils_qvto;
  
  @Inject
  private MetaModel xptMetaModel;
  
  @Inject
  private DiagramUpdater xptDiagramUpdater;
  
  @Inject
  private VisualIDRegistry xptVisualIDRegistry;
  
  @Inject
  private domain2notation xptDomain2notation;
  
  @Inject
  private NodeDescriptor nodeDescriptor;
  
  @Inject
  private LinkDescriptor linkDescriptor;
  
  public CharSequence body(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _attributes = this.attributes(it);
    _builder.append(_attributes);
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _refreshOnActivateMethod = this.refreshOnActivateMethod(it);
    _builder.append(_refreshOnActivateMethod);
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _featuresToSynchronizeMethod = this.getFeaturesToSynchronizeMethod(it);
    _builder.append(_featuresToSynchronizeMethod);
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _semanticChildrenListMethod = this.getSemanticChildrenListMethod(it);
    _builder.append(_semanticChildrenListMethod);
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _isOrphanedMethod = this.isOrphanedMethod(it);
    _builder.append(_isOrphanedMethod);
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _isMyDiagramElementMethod = this.isMyDiagramElementMethod(it);
    _builder.append(_isMyDiagramElementMethod);
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _isShortcutMethod = this.isShortcutMethod(it);
    _builder.append(_isShortcutMethod);
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence attributes(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      int _size = this._utils_qvto.getSemanticChildrenContainmentFeatures(it).size();
      boolean _greaterThan = (_size > 1);
      if (_greaterThan) {
        _builder.newLine();
        CharSequence _generatedMemberComment = this._common.generatedMemberComment();
        _builder.append(_generatedMemberComment);
        _builder.newLineIfNotEmpty();
        _builder.append("private java.util.Set<org.eclipse.emf.ecore.EStructuralFeature> myFeaturesToSynchronize;");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  /**
   * The canonical update mechanism of the GMF Runtime heavily depends on EditPart listeners being registered
   * prior to actual canonical update (thus, any change from this policy gets known to the interested EditParts.
   * NodeEditPart.activate() installs a transaction post-commit listener that refreshes the edit part. Canonical EditPolicy
   * is installed on a top (diagram) element, and its activation happens before diagram children get activated (and have a chance
   * to install their listeners). Hence this what I believe to be an questionable hack (at least, GEF's
   * activation sequence gets violated). See bugzilla https://bugs.eclipse.org/bugs/show_bug.cgi?id=314670
   */
  public CharSequence refreshOnActivateMethod(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected void refreshOnActivate() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// Need to activate editpart children before invoking the canonical refresh for EditParts to add event listeners");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("java.util.List<?> c = getHost().getChildren();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (int i = 0; i < c.size(); i++) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("((org.eclipse.gef.EditPart) c.get(i)).activate();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("super.refreshOnActivate();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getFeaturesToSynchronizeMethod(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      int _size = this._utils_qvto.getSemanticChildrenChildFeatures(it).size();
      boolean _equals = (_size == 1);
      if (_equals) {
        _builder.newLine();
        _builder.append("\t");
        CharSequence _generatedMemberComment = this._common.generatedMemberComment();
        _builder.append(_generatedMemberComment, "\t");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("protected org.eclipse.emf.ecore.EStructuralFeature getFeatureToSynchronize() {");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("return ");
        CharSequence _MetaFeature = this.xptMetaModel.MetaFeature(IterableExtensions.<GenFeature>head(this._utils_qvto.getSemanticChildrenContainmentFeatures(it)));
        _builder.append(_MetaFeature, "\t\t");
        _builder.append(";");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      } else {
        int _size_1 = this._utils_qvto.getSemanticChildrenChildFeatures(it).size();
        boolean _greaterThan = (_size_1 > 1);
        if (_greaterThan) {
          _builder.newLine();
          _builder.append("\t");
          CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
          _builder.append(_generatedMemberComment_1, "\t");
          _builder.newLineIfNotEmpty();
          _builder.append("\t");
          _builder.append("protected java.util.Set<org.eclipse.emf.ecore.EStructuralFeature> getFeaturesToSynchronize() {");
          _builder.newLine();
          _builder.append("\t\t");
          _builder.append("if (myFeaturesToSynchronize == null) {");
          _builder.newLine();
          _builder.append("\t\t\t");
          _builder.append("myFeaturesToSynchronize = new java.util.HashSet<org.eclipse.emf.ecore.EStructuralFeature>();");
          _builder.newLine();
          {
            Set<GenFeature> _semanticChildrenContainmentFeatures = this._utils_qvto.getSemanticChildrenContainmentFeatures(it);
            for(final GenFeature f : _semanticChildrenContainmentFeatures) {
              _builder.append("\t\t\t");
              CharSequence _addContainmentFeature = this.addContainmentFeature(f);
              _builder.append(_addContainmentFeature, "\t\t\t");
              _builder.newLineIfNotEmpty();
            }
          }
          _builder.append("\t\t");
          _builder.append("}");
          _builder.newLine();
          _builder.append("\t\t");
          _builder.append("return myFeaturesToSynchronize;");
          _builder.newLine();
          _builder.append("\t");
          _builder.append("}");
          _builder.newLine();
        }
      }
    }
    return _builder;
  }
  
  public CharSequence getSemanticChildrenListMethod(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected java.util.List<org.eclipse.emf.ecore.EObject> getSemanticChildrenList() {");
    _builder.newLine();
    {
      boolean _hasSemanticChildren = this._utils_qvto.hasSemanticChildren(it);
      if (_hasSemanticChildren) {
        _builder.append("\t");
        _builder.append("org.eclipse.gmf.runtime.notation.View viewObject = (org.eclipse.gmf.runtime.notation.View) getHost().getModel();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("java.util.LinkedList<org.eclipse.emf.ecore.EObject> result = new java.util.LinkedList<org.eclipse.emf.ecore.EObject>();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("java.util.List<");
        CharSequence _qualifiedClassName = this.nodeDescriptor.qualifiedClassName(it.getDiagram().getEditorGen().getDiagramUpdater());
        _builder.append(_qualifiedClassName, "\t");
        _builder.append("> childDescriptors = ");
        CharSequence _semanticChildrenMethodCall = this.xptDiagramUpdater.getSemanticChildrenMethodCall(it);
        _builder.append(_semanticChildrenMethodCall, "\t");
        _builder.append("(viewObject);");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("for (");
        CharSequence _qualifiedClassName_1 = this.nodeDescriptor.qualifiedClassName(it.getDiagram().getEditorGen().getDiagramUpdater());
        _builder.append(_qualifiedClassName_1, "\t");
        _builder.append(" d : childDescriptors) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("result.add(d.getModelElement());");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("return result;");
        _builder.newLine();
      } else {
        _builder.append("\t");
        _builder.append("return java.util.Collections.EMPTY_LIST;");
        _builder.newLine();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public boolean isDiagramThatContainsShortcurs(final GenContainerBase it) {
    return (this._common_qvto.oclIsKindOf(it, GenDiagram.class) && this._common_qvto.<String>notEmpty(((GenDiagram) it).getContainsShortcutsTo()));
  }
  
  protected CharSequence _isShortcutMethod(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  protected CharSequence _isShortcutMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      boolean _notEmpty = this._common_qvto.<String>notEmpty(it.getContainsShortcutsTo());
      if (_notEmpty) {
        CharSequence _generatedMemberComment = this._common.generatedMemberComment();
        _builder.append(_generatedMemberComment);
        _builder.newLineIfNotEmpty();
        _builder.append("protected static boolean isShortcut(org.eclipse.gmf.runtime.notation.View view) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("return view.getEAnnotation(\"Shortcut\") != null; ");
        CharSequence _nonNLS = this._common.nonNLS(1);
        _builder.append(_nonNLS, "\t");
        _builder.newLineIfNotEmpty();
        _builder.append("}");
        _builder.newLine();
      }
    }
    return _builder;
  }
  
  public CharSequence isOrphanedMethod(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected boolean isOrphaned(java.util.Collection<org.eclipse.emf.ecore.EObject> semanticChildren, final org.eclipse.gmf.runtime.notation.View view) {");
    _builder.newLine();
    {
      boolean _isDiagramThatContainsShortcurs = this.isDiagramThatContainsShortcurs(it);
      if (_isDiagramThatContainsShortcurs) {
        _builder.append("\t");
        _builder.append("if (isShortcut(view)) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("return ");
        CharSequence _qualifiedClassName = this.xptDiagramUpdater.qualifiedClassName(it.getDiagram().getEditorGen().getDiagramUpdater());
        _builder.append(_qualifiedClassName, "\t\t");
        _builder.append(".isShortcutOrphaned(view);");
        _builder.newLineIfNotEmpty();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.append("return isMyDiagramElement(view) && !semanticChildren.contains(view.getElement());");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence isMyDiagramElementMethod(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private boolean isMyDiagramElement(org.eclipse.gmf.runtime.notation.View view) {");
    _builder.newLine();
    _builder.append("\t");
    int semanticChildrenSize = IterableExtensions.size(this._utils_qvto.getSemanticChildren(it));
    _builder.newLineIfNotEmpty();
    {
      if ((semanticChildrenSize == 0)) {
        _builder.append("\t");
        _builder.append("return false;");
        _builder.newLine();
      } else {
        if ((semanticChildrenSize == 1)) {
          _builder.append("\t");
          _builder.append("return ");
          CharSequence _visualID = VisualIDRegistry.visualID(IterableExtensions.<GenNode>head(this._utils_qvto.getSemanticChildren(it)));
          _builder.append(_visualID, "\t");
          _builder.append(" == ");
          CharSequence _visualIDMethodCall = this.xptVisualIDRegistry.getVisualIDMethodCall(it.getDiagram());
          _builder.append(_visualIDMethodCall, "\t");
          _builder.append("(view);");
          _builder.newLineIfNotEmpty();
        } else {
          _builder.append("\t");
          _builder.append("int visualID = ");
          CharSequence _visualIDMethodCall_1 = this.xptVisualIDRegistry.getVisualIDMethodCall(it.getDiagram());
          _builder.append(_visualIDMethodCall_1, "\t");
          _builder.append("(view);");
          _builder.newLineIfNotEmpty();
          {
            if ((semanticChildrenSize > 3)) {
              _builder.append("\t");
              _builder.append("switch (visualID) {");
              _builder.newLine();
              {
                Iterable<GenNode> _semanticChildren = this._utils_qvto.getSemanticChildren(it);
                for(final GenNode ch : _semanticChildren) {
                  _builder.append("\t");
                  _builder.append("\t");
                  _builder.append("case ");
                  CharSequence _visualID_1 = VisualIDRegistry.visualID(ch);
                  _builder.append(_visualID_1, "\t\t");
                  _builder.append(":");
                  _builder.newLineIfNotEmpty();
                }
              }
              _builder.append("\t");
              _builder.append("\t\t");
              _builder.append("return true;");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("}");
              _builder.newLine();
              _builder.append("\t");
              _builder.append("return false;");
              _builder.newLine();
            } else {
              _builder.append("\t");
              _builder.append("return visualID == ");
              {
                Iterable<GenNode> _semanticChildren_1 = this._utils_qvto.getSemanticChildren(it);
                boolean _hasElements = false;
                for(final GenNode ch_1 : _semanticChildren_1) {
                  if (!_hasElements) {
                    _hasElements = true;
                  } else {
                    _builder.appendImmediate(" || visualID == ", "\t");
                  }
                  CharSequence _visualID_2 = VisualIDRegistry.visualID(ch_1);
                  _builder.append(_visualID_2, "\t");
                }
              }
              _builder.append("; ");
              _builder.newLineIfNotEmpty();
            }
          }
        }
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence refreshSemanticChildren(final GenContainerBase it, final String createdViewsVar, final GenDiagramUpdater diagramUpdater) {
    StringConcatenation _builder = new StringConcatenation();
    boolean childrenShareSameMetaclass = this._utils_qvto.hasConformableSemanticChildren(it);
    _builder.newLineIfNotEmpty();
    _builder.append("java.util.List<");
    CharSequence _qualifiedClassName = this.nodeDescriptor.qualifiedClassName(diagramUpdater);
    _builder.append(_qualifiedClassName);
    _builder.append("> childDescriptors = ");
    {
      boolean _hasSemanticChildren = this._utils_qvto.hasSemanticChildren(it);
      if (_hasSemanticChildren) {
        _builder.newLineIfNotEmpty();
        CharSequence _semanticChildrenMethodCall = this.xptDiagramUpdater.getSemanticChildrenMethodCall(it);
        _builder.append(_semanticChildrenMethodCall);
        _builder.append("((org.eclipse.gmf.runtime.notation.View) getHost().getModel());");
        _builder.newLineIfNotEmpty();
      } else {
        _builder.append("/* see #352271 */ java.util.Collections.emptyList();");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.append("java.util.LinkedList<org.eclipse.gmf.runtime.notation.View> orphaned = new java.util.LinkedList<org.eclipse.gmf.runtime.notation.View>(); ");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// we care to check only views we recognize as ours");
    {
      boolean _isDiagramThatContainsShortcurs = this.isDiagramThatContainsShortcurs(it);
      if (_isDiagramThatContainsShortcurs) {
        _builder.append(" and not shortcuts");
      }
    }
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("java.util.LinkedList<org.eclipse.gmf.runtime.notation.View> knownViewChildren = new java.util.LinkedList<org.eclipse.gmf.runtime.notation.View>();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (org.eclipse.gmf.runtime.notation.View v : getViewChildren()) {");
    _builder.newLine();
    {
      boolean _isDiagramThatContainsShortcurs_1 = this.isDiagramThatContainsShortcurs(it);
      if (_isDiagramThatContainsShortcurs_1) {
        _builder.append("\t\t");
        _builder.append("if (isShortcut(v)) {");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("if (");
        CharSequence _qualifiedClassName_1 = this.xptDiagramUpdater.qualifiedClassName(it.getDiagram().getEditorGen().getDiagramUpdater());
        _builder.append(_qualifiedClassName_1, "\t\t\t");
        _builder.append(".isShortcutOrphaned(v)) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t\t");
        _builder.append("orphaned.add(v);");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("continue;");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("} ");
        _builder.newLine();
      }
    }
    _builder.append("\t\t");
    _builder.append("if (isMyDiagramElement(v)) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("knownViewChildren.add(v);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// alternative to #cleanCanonicalSemanticChildren(getViewChildren(), semanticChildren)");
    _builder.newLine();
    {
      if (childrenShareSameMetaclass) {
        _builder.append("\t");
        _builder.append("java.util.HashMap<");
        CharSequence _qualifiedClassName_2 = this.nodeDescriptor.qualifiedClassName(diagramUpdater);
        _builder.append(_qualifiedClassName_2, "\t");
        _builder.append(", java.util.LinkedList<org.eclipse.gmf.runtime.notation.View>> potentialViews = new java.util.HashMap<");
        CharSequence _qualifiedClassName_3 = this.nodeDescriptor.qualifiedClassName(diagramUpdater);
        _builder.append(_qualifiedClassName_3, "\t");
        _builder.append(", java.util.LinkedList<org.eclipse.gmf.runtime.notation.View>>(); ");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    _builder.append("//");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// iteration happens over list of desired semantic elements, trying to find best matching View, while original CEP");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// iterates views, potentially losing view (size/bounds) information - i.e. if there are few views to reference same EObject, only last one ");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// to answer isOrphaned == true will be used for the domain element representation, see #cleanCanonicalSemanticChildren()");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (java.util.Iterator<");
    String _nodeDescriptorQualifiedClassName = diagramUpdater.getNodeDescriptorQualifiedClassName();
    _builder.append(_nodeDescriptorQualifiedClassName, "\t");
    _builder.append("> descriptorsIterator = childDescriptors.iterator(); descriptorsIterator.hasNext();) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    CharSequence _qualifiedClassName_4 = this.nodeDescriptor.qualifiedClassName(diagramUpdater);
    _builder.append(_qualifiedClassName_4, "\t\t");
    _builder.append(" next = descriptorsIterator.next();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("String hint = ");
    CharSequence _typeMethodCall = this.xptVisualIDRegistry.typeMethodCall(it, "next.getVisualID()");
    _builder.append(_typeMethodCall, "\t\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("java.util.LinkedList<org.eclipse.gmf.runtime.notation.View> perfectMatch = new java.util.LinkedList<org.eclipse.gmf.runtime.notation.View>(); // both semanticElement and hint match that of NodeDescriptor");
    _builder.newLine();
    {
      if (childrenShareSameMetaclass) {
        _builder.append("\t\t");
        _builder.append("java.util.LinkedList<org.eclipse.gmf.runtime.notation.View> potentialMatch = new java.util.LinkedList<org.eclipse.gmf.runtime.notation.View>();  // semanticElement matches, hint does not");
        _builder.newLine();
      }
    }
    _builder.append("\t\t");
    _builder.append("for (org.eclipse.gmf.runtime.notation.View childView : getViewChildren()) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("org.eclipse.emf.ecore.EObject semanticElement = childView.getElement();");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("if (next.getModelElement().equals(semanticElement)) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("if (hint.equals(childView.getType())) {");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("perfectMatch.add(childView);");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("// actually, can stop iteration over view children here, but");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("// may want to use not the first view but last one as a \'real\' match (the way original CEP does");
    _builder.newLine();
    _builder.append("\t\t\t\t\t");
    _builder.append("// with its trick with viewToSemanticMap inside #cleanCanonicalSemanticChildren");
    _builder.newLine();
    {
      if (childrenShareSameMetaclass) {
        _builder.append("\t\t\t\t");
        _builder.append("} else {");
        _builder.newLine();
        _builder.append("\t\t\t\t");
        _builder.append("\t");
        _builder.append("potentialMatch.add(childView);");
        _builder.newLine();
      }
    }
    _builder.append("\t\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (perfectMatch.size() > 0) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("descriptorsIterator.remove(); // precise match found no need to create anything for the NodeDescriptor");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("// use only one view (first or last?), keep rest as orphaned for further consideration");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("knownViewChildren.remove(perfectMatch.getFirst());");
    _builder.newLine();
    {
      if (childrenShareSameMetaclass) {
        _builder.append("\t\t");
        _builder.append("} else if (potentialMatch.size() > 0) {");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("potentialViews.put(next, potentialMatch); ");
        _builder.newLine();
      }
    }
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// those left in knownViewChildren are subject to removal - they are our diagram elements we didn\'t find match to,");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("// or those we have potential matches to, and thus need to be recreated, preserving size/location information.");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("orphaned.addAll(knownViewChildren);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("//");
    _builder.newLine();
    {
      if (childrenShareSameMetaclass) {
        _builder.append("\t");
        _builder.append("org.eclipse.gmf.runtime.emf.commands.core.command.CompositeTransactionalCommand boundsCommand = new org.eclipse.gmf.runtime.emf.commands.core.command.CompositeTransactionalCommand(host().getEditingDomain(), org.eclipse.gmf.runtime.diagram.ui.l10n.DiagramUIMessages.SetLocationCommand_Label_Resize);");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.append("java.util.ArrayList<org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest.ViewDescriptor> viewDescriptors = new java.util.ArrayList<org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest.ViewDescriptor>(childDescriptors.size());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (");
    CharSequence _qualifiedClassName_5 = this.nodeDescriptor.qualifiedClassName(diagramUpdater);
    _builder.append(_qualifiedClassName_5, "\t");
    _builder.append(" next : childDescriptors) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("String hint = ");
    CharSequence _typeMethodCall_1 = this.xptVisualIDRegistry.typeMethodCall(it, "next.getVisualID()");
    _builder.append(_typeMethodCall_1, "\t\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.core.runtime.IAdaptable elementAdapter = new CanonicalElementAdapter(next.getModelElement(), hint);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest.ViewDescriptor descriptor = new org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest.ViewDescriptor(elementAdapter, org.eclipse.gmf.runtime.notation.Node.class, hint, org.eclipse.gmf.runtime.diagram.core.util.ViewUtil.APPEND, false, host().getDiagramPreferencesHint());");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("viewDescriptors.add(descriptor);");
    _builder.newLine();
    {
      if (childrenShareSameMetaclass) {
        _builder.append("\t");
        _builder.append("java.util.LinkedList<org.eclipse.gmf.runtime.notation.View> possibleMatches = potentialViews.get(next);");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("if (possibleMatches != null) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("// from potential matches, leave those that were not eventually used for some other NodeDescriptor (i.e. those left as orphaned)");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("possibleMatches.retainAll(knownViewChildren);");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("if (possibleMatches != null && !possibleMatches.isEmpty()) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("org.eclipse.gmf.runtime.notation.View originalView = possibleMatches.getFirst();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("knownViewChildren.remove(originalView); // remove not to copy properties of the same view again and again");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("// add command to copy properties");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("if (originalView instanceof org.eclipse.gmf.runtime.notation.Node) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t");
        _builder.append("if (((org.eclipse.gmf.runtime.notation.Node) originalView).getLayoutConstraint() instanceof org.eclipse.gmf.runtime.notation.Bounds) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t\t");
        _builder.append("org.eclipse.gmf.runtime.notation.Bounds b = (org.eclipse.gmf.runtime.notation.Bounds) ((org.eclipse.gmf.runtime.notation.Node) originalView).getLayoutConstraint();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t\t");
        _builder.append("boundsCommand.add(new org.eclipse.gmf.runtime.diagram.ui.commands.SetBoundsCommand(boundsCommand.getEditingDomain(), boundsCommand.getLabel(), descriptor, new org.eclipse.draw2d.geometry.Rectangle(b.getX(), b.getY(), b.getWidth(), b.getHeight())));");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t");
        _builder.append("} else if (((org.eclipse.gmf.runtime.notation.Node) originalView).getLayoutConstraint() instanceof org.eclipse.gmf.runtime.notation.Location) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t\t");
        _builder.append("org.eclipse.gmf.runtime.notation.Location l = (org.eclipse.gmf.runtime.notation.Location) ((org.eclipse.gmf.runtime.notation.Node) originalView).getLayoutConstraint();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t\t");
        _builder.append("boundsCommand.add(new org.eclipse.gmf.runtime.diagram.ui.commands.SetBoundsCommand(boundsCommand.getEditingDomain(), boundsCommand.getLabel(), descriptor, new org.eclipse.draw2d.geometry.Point(l.getX(), l.getY())));");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t");
        _builder.append("} else if (((org.eclipse.gmf.runtime.notation.Node) originalView).getLayoutConstraint() instanceof org.eclipse.gmf.runtime.notation.Size) {");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t\t");
        _builder.append("org.eclipse.gmf.runtime.notation.Size s = (org.eclipse.gmf.runtime.notation.Size) ((org.eclipse.gmf.runtime.notation.Node) originalView).getLayoutConstraint();");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t\t");
        _builder.append("boundsCommand.add(new org.eclipse.gmf.runtime.diagram.ui.commands.SetBoundsCommand(boundsCommand.getEditingDomain(), boundsCommand.getLabel(), descriptor, new org.eclipse.draw2d.geometry.Dimension(s.getWidth(), s.getHeight())));");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
        _builder.append("\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    _builder.append("boolean changed = deleteViews(orphaned.iterator());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("//");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.requests.CreateViewRequest request = getCreateViewRequest(viewDescriptors);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gef.commands.Command cmd = getCreateViewCommand(request);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (cmd != null && cmd.canExecute() ) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.commands.SetViewMutabilityCommand.makeMutable(new org.eclipse.gmf.runtime.emf.core.util.EObjectAdapter(host().getNotationView())).execute();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("executeCommand(cmd);");
    _builder.newLine();
    _builder.append("\t\t");
    {
      if (childrenShareSameMetaclass) {
        _builder.append("if (boundsCommand.canExecute()) {");
        _builder.newLineIfNotEmpty();
        _builder.append("\t\t");
        _builder.append("\t");
        _builder.append("executeCommand(new org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy(boundsCommand.reduce()));");
        _builder.newLine();
        _builder.append("\t\t");
        _builder.append("}");
        _builder.newLine();
      }
    }
    _builder.append("\t\t");
    _builder.append("@SuppressWarnings(\"unchecked\")");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("java.util.List<org.eclipse.core.runtime.IAdaptable> nl = (java.util.List<org.eclipse.core.runtime.IAdaptable>) request.getNewObject();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append(createdViewsVar, "\t\t");
    _builder.append(".addAll(nl);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (changed || ");
    _builder.append(createdViewsVar, "\t");
    _builder.append(".size() > 0) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("postProcessRefreshSemantic(createdViews);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence executeLayoutCommand(final GenContainerBase it, final String createdViewsVar) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("if (");
    _builder.append(createdViewsVar);
    _builder.append(".size() > 1) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("// perform a layout of the container");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.commands.DeferredLayoutCommand layoutCmd = new org.eclipse.gmf.runtime.diagram.ui.commands.DeferredLayoutCommand(host().getEditingDomain(), ");
    _builder.append(createdViewsVar, "\t");
    _builder.append(", host());");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("executeCommand(new org.eclipse.gmf.runtime.diagram.ui.commands.ICommandProxy(layoutCmd));");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence refreshConnectionsBody(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _Domain2Notation = this.Domain2Notation(it);
    _builder.append(_Domain2Notation);
    _builder.append(" domain2NotationMap = new ");
    CharSequence _Domain2Notation_1 = this.Domain2Notation(it);
    _builder.append(_Domain2Notation_1);
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    _builder.append("java.util.Collection<");
    CharSequence _qualifiedClassName = this.linkDescriptor.qualifiedClassName(it.getEditorGen().getDiagramUpdater());
    _builder.append(_qualifiedClassName);
    _builder.append("> linkDescriptors = collectAllLinks(getDiagram(), domain2NotationMap);");
    _builder.newLineIfNotEmpty();
    _builder.append("java.util.List<org.eclipse.gmf.runtime.notation.View> edges = new java.util.ArrayList<org.eclipse.gmf.runtime.notation.View>();");
    _builder.newLine();
    _builder.append("for (Object edge : getDiagram().getEdges())");
    _builder.newLine();
    _builder.append("{");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (edge instanceof org.eclipse.gmf.runtime.notation.View)");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("{");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("edges.add((org.eclipse.gmf.runtime.notation.View) edge);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.append("java.util.Collection<org.eclipse.gmf.runtime.notation.View> existingLinks = new java.util.LinkedList<org.eclipse.gmf.runtime.notation.View>(edges);");
    _builder.newLine();
    _builder.append("for (java.util.Iterator<org.eclipse.gmf.runtime.notation.View> linksIterator = existingLinks.iterator(); linksIterator.hasNext();) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.notation.Edge nextDiagramLink = (org.eclipse.gmf.runtime.notation.Edge) linksIterator.next();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("int diagramLinkVisualID = ");
    CharSequence _visualIDMethodCall = this.xptVisualIDRegistry.getVisualIDMethodCall(it);
    _builder.append(_visualIDMethodCall, "\t");
    _builder.append("(nextDiagramLink);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (diagramLinkVisualID == -1");
    {
      final Function1<GenLink, Boolean> _function = new Function1<GenLink, Boolean>() {
        @Override
        public Boolean apply(final GenLink gl) {
          LinkModelFacet _modelFacet = gl.getModelFacet();
          return Boolean.valueOf((_modelFacet == null));
        }
      };
      Iterable<GenLink> _filter = IterableExtensions.<GenLink>filter(it.getLinks(), _function);
      for(final GenLink link : _filter) {
        CharSequence _compareLinkVisualID = this.compareLinkVisualID(link);
        _builder.append(_compareLinkVisualID, "\t");
      }
    }
    _builder.append(") {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("if (nextDiagramLink.getSource() != null && nextDiagramLink.getTarget() != null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("linksIterator.remove();");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("continue;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.emf.ecore.EObject diagramLinkObject = nextDiagramLink.getElement();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.emf.ecore.EObject diagramLinkSrc = nextDiagramLink.getSource().getElement();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.emf.ecore.EObject diagramLinkDst = nextDiagramLink.getTarget().getElement();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (java.util.Iterator<");
    String _linkDescriptorQualifiedClassName = it.getEditorGen().getDiagramUpdater().getLinkDescriptorQualifiedClassName();
    _builder.append(_linkDescriptorQualifiedClassName, "\t");
    _builder.append("> linkDescriptorsIterator = linkDescriptors.iterator(); linkDescriptorsIterator.hasNext();) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    CharSequence _qualifiedClassName_1 = this.linkDescriptor.qualifiedClassName(it.getEditorGen().getDiagramUpdater());
    _builder.append(_qualifiedClassName_1, "\t\t");
    _builder.append(" nextLinkDescriptor = linkDescriptorsIterator.next();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("if (diagramLinkObject == nextLinkDescriptor.getModelElement() && diagramLinkSrc == nextLinkDescriptor.getSource() && diagramLinkDst == nextLinkDescriptor.getDestination() && diagramLinkVisualID == nextLinkDescriptor.getVisualID()) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("linksIterator.remove();");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("linkDescriptorsIterator.remove();");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("break;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.append("deleteViews(existingLinks.iterator());");
    _builder.newLine();
    _builder.append("return createConnections(linkDescriptors, domain2NotationMap);");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence refreshConnectionsAuxMethods(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _collectAllLinksMethod = this.collectAllLinksMethod(it);
    _builder.append(_collectAllLinksMethod);
    _builder.newLineIfNotEmpty();
    CharSequence _createConnectionsMethod = this.createConnectionsMethod(it);
    _builder.append(_createConnectionsMethod);
    _builder.newLineIfNotEmpty();
    CharSequence _editPartMethod = this.getEditPartMethod(it);
    _builder.append(_editPartMethod);
    _builder.newLineIfNotEmpty();
    CharSequence _diagramMethod = this.getDiagramMethod(it);
    _builder.append(_diagramMethod);
    _builder.newLineIfNotEmpty();
    CharSequence _sourceEditPartMethod = this.getSourceEditPartMethod(it);
    _builder.append(_sourceEditPartMethod);
    _builder.newLineIfNotEmpty();
    CharSequence _targetEditPartMethod = this.getTargetEditPartMethod(it);
    _builder.append(_targetEditPartMethod);
    _builder.newLineIfNotEmpty();
    CharSequence _hintedEditPartMethod = this.getHintedEditPartMethod(it);
    _builder.append(_hintedEditPartMethod);
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence collectAllLinksMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private java.util.Collection<");
    CharSequence _qualifiedClassName = this.linkDescriptor.qualifiedClassName(it.getEditorGen().getDiagramUpdater());
    _builder.append(_qualifiedClassName);
    _builder.append("> collectAllLinks(org.eclipse.gmf.runtime.notation.View view, ");
    CharSequence _Domain2Notation = this.Domain2Notation(it);
    _builder.append(_Domain2Notation);
    _builder.append(" domain2NotationMap) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (!");
    CharSequence _modelID = VisualIDRegistry.modelID(it);
    _builder.append(_modelID, "\t");
    _builder.append(".equals(");
    CharSequence _modelIDMethodCall = this.xptVisualIDRegistry.getModelIDMethodCall(it);
    _builder.append(_modelIDMethodCall, "\t");
    _builder.append("(view))) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("return java.util.Collections.emptyList();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("java.util.LinkedList<");
    CharSequence _qualifiedClassName_1 = this.linkDescriptor.qualifiedClassName(it.getEditorGen().getDiagramUpdater());
    _builder.append(_qualifiedClassName_1, "\t");
    _builder.append("> result = new java.util.LinkedList<");
    CharSequence _qualifiedClassName_2 = this.linkDescriptor.qualifiedClassName(it.getEditorGen().getDiagramUpdater());
    _builder.append(_qualifiedClassName_2, "\t");
    _builder.append(">();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("String vid = ");
    CharSequence _visualIDMethodCall = this.xptVisualIDRegistry.getVisualIDMethodCall(it);
    _builder.append(_visualIDMethodCall, "\t");
    _builder.append("(view);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (vid != null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("switch (vid) {");
    _builder.newLine();
    {
      Iterable<GenCommonBase> _allSemanticElements = this._utils_qvto.getAllSemanticElements(it);
      for(final GenCommonBase se : _allSemanticElements) {
        _builder.append("\t\t\t");
        CharSequence _caseSemanticElement = this.caseSemanticElement(se);
        _builder.append(_caseSemanticElement, "\t\t\t");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (java.util.Iterator<?> children = view.getChildren().iterator(); children.hasNext();) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("result.addAll(collectAllLinks((org.eclipse.gmf.runtime.notation.View) children.next(), domain2NotationMap));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (java.util.Iterator<?> edges = view.getSourceEdges().iterator(); edges.hasNext();) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("result.addAll(collectAllLinks((org.eclipse.gmf.runtime.notation.View) edges.next(), domain2NotationMap));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return result;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence createConnectionsMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private java.util.Collection<org.eclipse.core.runtime.IAdaptable> createConnections(java.util.Collection<");
    CharSequence _qualifiedClassName = this.linkDescriptor.qualifiedClassName(it.getEditorGen().getDiagramUpdater());
    _builder.append(_qualifiedClassName);
    _builder.append("> linkDescriptors, ");
    CharSequence _Domain2Notation = this.Domain2Notation(it);
    _builder.append(_Domain2Notation);
    _builder.append(" domain2NotationMap) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("java.util.LinkedList<org.eclipse.core.runtime.IAdaptable> adapters = new java.util.LinkedList<org.eclipse.core.runtime.IAdaptable>();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("for (");
    CharSequence _qualifiedClassName_1 = this.linkDescriptor.qualifiedClassName(it.getEditorGen().getDiagramUpdater());
    _builder.append(_qualifiedClassName_1, "\t");
    _builder.append(" nextLinkDescriptor : linkDescriptors) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gef.EditPart sourceEditPart = getSourceEditPart(nextLinkDescriptor, domain2NotationMap);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gef.EditPart targetEditPart = getTargetEditPart(nextLinkDescriptor, domain2NotationMap);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (sourceEditPart == null || targetEditPart == null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("continue;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.requests.CreateConnectionViewRequest.ConnectionViewDescriptor descriptor = new org.eclipse.gmf.runtime.diagram.ui.requests.CreateConnectionViewRequest.ConnectionViewDescriptor(nextLinkDescriptor.getSemanticAdapter(), ");
    CharSequence _typeMethodCall = this.xptVisualIDRegistry.typeMethodCall(it, "nextLinkDescriptor.getVisualID()");
    _builder.append(_typeMethodCall, "\t\t");
    _builder.append(", org.eclipse.gmf.runtime.diagram.core.util.ViewUtil.APPEND, false, ((org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart) getHost()).getDiagramPreferencesHint());");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.ui.requests.CreateConnectionViewRequest ccr = new org.eclipse.gmf.runtime.diagram.ui.requests.CreateConnectionViewRequest(descriptor);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("ccr.setType(org.eclipse.gmf.runtime.diagram.ui.requests.RequestConstants.REQ_CONNECTION_START);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("ccr.setSourceEditPart(sourceEditPart);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("sourceEditPart.getCommand(ccr);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("ccr.setTargetEditPart(targetEditPart);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("ccr.setType(org.eclipse.gmf.runtime.diagram.ui.requests.RequestConstants.REQ_CONNECTION_END);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.gef.commands.Command cmd = targetEditPart.getCommand(ccr);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (cmd != null && cmd.canExecute()) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("executeCommand(cmd);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("org.eclipse.core.runtime.IAdaptable viewAdapter = (org.eclipse.core.runtime.IAdaptable) ccr.getNewObject();");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("if (viewAdapter != null) {");
    _builder.newLine();
    _builder.append("\t\t\t\t");
    _builder.append("adapters.add(viewAdapter);");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return adapters;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getEditPartMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.gef.EditPart getEditPart(org.eclipse.emf.ecore.EObject domainModelElement, ");
    CharSequence _Domain2Notation = this.Domain2Notation(it);
    _builder.append(_Domain2Notation);
    _builder.append(" domain2NotationMap) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.notation.View view = domain2NotationMap.get(domainModelElement);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (view != null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return (org.eclipse.gef.EditPart) getHost().getViewer().getEditPartRegistry().get(view);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getHintedEditPartMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected final org.eclipse.gef.EditPart getHintedEditPart(org.eclipse.emf.ecore.EObject domainModelElement, ");
    CharSequence _Domain2Notation = this.Domain2Notation(it);
    _builder.append(_Domain2Notation);
    _builder.append(" domain2NotationMap, int hintVisualId) { ");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.notation.View view = domain2NotationMap.getHinted(domainModelElement, ");
    CharSequence _typeMethodCall = this.xptVisualIDRegistry.typeMethodCall(it, "hintVisualId");
    _builder.append(_typeMethodCall, "\t");
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (view != null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return (org.eclipse.gef.EditPart) getHost().getViewer().getEditPartRegistry().get(view);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getSourceEditPartMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("\t");
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.gef.EditPart getSourceEditPart(org.eclipse.gmf.tooling.runtime.update.UpdaterLinkDescriptor descriptor, ");
    CharSequence _Domain2Notation = this.Domain2Notation(it);
    _builder.append(_Domain2Notation);
    _builder.append(" domain2NotationMap) { ");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return getEditPart(descriptor.getSource(), domain2NotationMap);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getTargetEditPartMethod(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("\t");
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.gef.EditPart getTargetEditPart(org.eclipse.gmf.tooling.runtime.update.UpdaterLinkDescriptor descriptor, ");
    CharSequence _Domain2Notation = this.Domain2Notation(it);
    _builder.append(_Domain2Notation);
    _builder.append(" domain2NotationMap) { ");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return getEditPart(descriptor.getDestination(), domain2NotationMap);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getDiagramMethod(final GenContainerBase it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.gmf.runtime.notation.Diagram getDiagram() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return ((org.eclipse.gmf.runtime.notation.View) getHost().getModel()).getDiagram();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  protected CharSequence Domain2Notation(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _DomainToNotationClassName = this.xptDomain2notation.DomainToNotationClassName(it);
    _builder.append(_DomainToNotationClassName);
    return _builder;
  }
  
  protected CharSequence addContainmentFeature(final GenFeature it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("myFeaturesToSynchronize.add(");
    CharSequence _MetaFeature = this.xptMetaModel.MetaFeature(it);
    _builder.append(_MetaFeature);
    _builder.append(");");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence compareLinkVisualID(final GenLink it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append(" ");
    _builder.append("|| diagramLinkVisualID == ");
    CharSequence _visualID = VisualIDRegistry.visualID(it);
    _builder.append(_visualID, " ");
    return _builder;
  }
  
  /**
   * GMF has no links to "leaf" child nodes.
   */
  protected CharSequence _caseSemanticElement(final GenChildLabelNode it) {
    StringConcatenation _builder = new StringConcatenation();
    return _builder;
  }
  
  protected CharSequence _caseSemanticElement(final GenCommonBase it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("case ");
    CharSequence _visualID = VisualIDRegistry.visualID(it);
    _builder.append(_visualID);
    _builder.append(": {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (!domain2NotationMap.containsKey(view.getElement())) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("result.addAll(");
    CharSequence _containedLinksMethodCall = this.xptDiagramUpdater.getContainedLinksMethodCall(it);
    _builder.append(_containedLinksMethodCall, "\t");
    _builder.append("(view));");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("domain2NotationMap.putView(view.getElement(), view);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("break;\t");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence isShortcutMethod(final GenContainerBase it) {
    if (it instanceof GenDiagram) {
      return _isShortcutMethod((GenDiagram)it);
    } else if (it != null) {
      return _isShortcutMethod(it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
  
  public CharSequence caseSemanticElement(final GenCommonBase it) {
    if (it instanceof GenChildLabelNode) {
      return _caseSemanticElement((GenChildLabelNode)it);
    } else if (it != null) {
      return _caseSemanticElement(it);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(it).toString());
    }
  }
}
